@@ -30,6 +30,7 @@ import fedmsg
 import fedmsg.encoding
 import fedmsg.meta
 from fedmsg.commands import BaseCommand
+from fedmsg.utils import cowsay_output
 
 
 class TailCommand(BaseCommand):
@@ -60,6 +61,12 @@ class TailCommand(BaseCommand):
             'default': False,
             'action': 'store_true',
         }),
+        (['--cowsay'], {
+            'dest': 'cowsay',
+            'help': 'Print cowsay output of messages',
+            'default': False,
+            'action': 'store_true',
+        }),
         (['--terse'], {
             'dest': 'terse',
             'help': 'Print "english" representations of messages only.',
@@ -137,6 +144,14 @@ class TailCommand(BaseCommand):
         if self.config['terse']:
             formatter = lambda d: "\n" + fedmsg.meta.msg2repr(d, **self.config)
 
+        if self.config['cowsay']:
+            def formatter(d):
+                result, error = cowsay_output(fedmsg.meta.msg2subtitle(d, **self.config))
+                if not error:
+                    return "\n" + result
+                else:
+                    return "\n" + error
+
         # Build regular expressions for use in our loop.
         exclusive_regexp = re.compile(self.config['exclusive_regexp'])
         inclusive_regexp = re.compile(self.config['inclusive_regexp'])
@@ -149,7 +164,7 @@ class TailCommand(BaseCommand):
             packages = set(map(str.strip, self.config['packages'].split(',')))
 
         # Only initialize this if we have to
-        if users or packages or self.config['terse']:
+        if users or packages or self.config['terse'] or self.config['cowsay']:
             fedmsg.meta.make_processors(**self.config)
 
         # Spin up a zmq.Poller and yield messages
